#!/usr/local/bin/perl

# HTML to PDF (via LaTeX)
# Designed to work within the constraints of the UKOLN qa focus briefing documents!
# Will dump any tags that it doesn't recognise

# Copyright july 2006 Emma Tonkin, e.tonkin@ukoln.ac.uk
use strict;
use HTML::TreeBuilder;
use HTML::TagFilter;
use Data::Dumper;
#use Date::Manip;
#use Time::ParseDate;
#use Time::Zone;
use XML::Atom;
use XML::Atom::Feed;
use XML::Atom::Entry;
use XML::Atom::Person;
use DateTime;
use HTML::Element;
use LWP::Simple;
use Digest::MD5;
use Image::Size;
no warnings 'utf8'; 

package PageObject;
# one page object per segment / html source page
=head1 
PageObject

Holds various representations of (and metadata about) a given page.
=cut

sub new{
	my $self={
		_uri=>undef,
		_author=>undef,
		_subject=>undef,
		_title=>undef,
		_base=>undef,
		_images=>undef,
		_updated=>undef,
		_description=>undef,
		_publish_date=>undef,
		_pagecontent=>undef, # page content in html 
		_pagecontenthtml=>undef, # page content in html 
		_pagecontenttex=>undef, # tex version of page content 

	};
    bless $self,'PageObject';
	return $self;
}

=head2
Access methods

An access method exists for every data element. These are:
=cut

sub images{
		my ( $self, @images) = @_;
		@{$self->{_images}} = @images if defined(@images);
		if (defined @images) {return @{$self->{_images}}} else {return 0};
}

sub pagecontenthtml{
		my ( $self, $pagecontenthtml) = @_;
		$self->{_pagecontenthtml} = $pagecontenthtml if defined($pagecontenthtml);
		return $self->{_pagecontenthtml};
}
sub pagecontenttex{
		my ( $self, $pagecontenttex) = @_;
		$self->{_pagecontenttex} = $pagecontenttex if defined($pagecontenttex);
		return $self->{_pagecontenttex};
}
sub pagecontent{
		my ( $self, $pagecontent) = @_;
		$self->{_pagecontent} = $pagecontent if defined($pagecontent);
		return $self->{_pagecontent};
}

sub updated{
        my ( $self, $updated) = @_;
        $self->{_updated} = $updated if defined($updated);
        return $self->{_updated};
}

sub publish_date{
        my ( $self, $publish_date) = @_;
        $self->{_publish_date} = $publish_date if defined($publish_date);
        return $self->{_publish_date};
}

sub description{
        my ( $self, $description) = @_;
        $self->{_description} = $description if defined($description);
        return $self->{_description};
}
sub uri{
        my ( $self, $uri) = @_;
        $self->{_uri} = $uri if defined($uri);
        return $self->{_uri};
}
sub subject{
        my ( $self, $subject) = @_;
        $self->{_subject} = $subject if defined($subject);
        return $self->{_subject};
}


sub author{
        my ( $self, $author) = @_;
        $self->{_author} = $author if defined($author);
        return $self->{_author};
}

sub title{
        my ( $self, $atitle) = @_;
        $self->{_title} = $atitle if defined($atitle);
        return $self->{_title};
}

sub base{
        my ( $self, $abase) = @_;
        $self->{_base} = $abase if defined($abase);
        return $self->{_base};
}

package HTMLtoTex;
=head1 
HTMLtoTex

An increasingly inaccurately named package initially designed to take an HTML page and provide a representation of that page in LaTeX. In future it is hoped to also transmit HTML to Textile2.
=cut

sub new{
        my $self={
				_pageobjects=>undef, # array of page objects 
                _uri=>undef,  # might actually contain several uris in total...?
				_author=>undef,
				_title=>undef, # refers to the final Tex! Not the segments!
                _updated=>undef,
                _published=>undef,
				_pagecontent=>undef,
				_ignoredivs=>undef,
				_tex_useimages=>undef,
				_tex_useglossary=>undef,
        };
        bless $self,'HTMLtoTex';
	return $self;
}

sub useimages{
	my ( $self, $value)=@_;
	$self->{_tex_useimages} = $value if defined ($value);
	return $self->{_tex_useimages};
}

sub useglossary{
	my ( $self, $value)=@_;
	$self->{_tex_useglossary} = $value if defined ($value);
	return $self->{_tex_useglossary};
}

sub uri{
        my ( $self, $uri) = @_;
        $self->{_uri} = $uri if defined($uri);
        return $self->{_uri};
}


sub ignoredivs{
=head2
ignoredivs: set id/class blacklist

In reality most pages are built up of a little content and a lot of cruft. Removing the cruft can be facilitated if you know the id or class of the container elements around it - if they are set here, the divs/paragraph elements with that id/class will be ignored entirely.
=cut 
		my ( $self, @ignoredivs) = @_;
		if($#ignoredivs>-1){
			 push(@{$self->{_ignoredivs}},@ignoredivs);
		}	
		if (defined @ignoredivs) {return @{$self->{_ignoredivs}}} else {return 0};
}

sub get_tex_chapter{
	my ($self, $pageobjectref)=@_;
	my @arr=@{$self->{_pageobjects}};
	
	my $wrapping="
\\documentclass[9pt, a4paper]{report}
\\begin{document}
		";
		if( -e 'header.conf'){
			open (FILE,'header.conf');
			$wrapping=join('',<FILE>);
			close FILE;
		}
		my $footer= "
\\end{document}
			";
			$wrapping.="\\chapter{ $pageobjectref ".$arr[$pageobjectref]->title()."}\n";
	return $wrapping.$arr[$pageobjectref]->pagecontenttex().$footer;
}

sub make_tex_all(){
 	my ($self)=@_;
	my @arr=@{$self->{_pageobjects}};
	my $wrapping="
\\documentclass[9pt, a4paper]{report}
\\begin{document}
		";
		if( -e 'header-book.conf'){
			open (FILE,'header-book.conf');
			$wrapping=join('',<FILE>);
			close FILE;
		}
		my $footer= "
\\end{document}
			"; 
		if( -e 'footer-book.conf'){
			open (FILE,'footer-book.conf');
			$footer=join('',<FILE>);
			close FILE;
		}
		my $content="";
		foreach my $pageobject (@arr){
			$content.="\\chapter{ ".$pageobject->title()." }";
			$content.=$pageobject->pagecontenttex();
		}
	# add glossary	
	if($self->{'_tex_useglossary'}>0){
		my %glossarydefs;
		foreach my $pageobject(@arr){
			foreach my $line (split(/\n/,$pageobject->pagecontenttex())){
				
				if($line=~/nomenclature\{(.*)\}\{(.*)\}/){
					# print "$1 $2 \n";
					my $temp=$1;
					$temp=~s/\<//g;
					$temp=~s/\>//g;
					$temp=~s/\$//g;
					$temp=~s/\ //g;
					if($2 eq "" || !$2){
					} else {
						$glossarydefs{$temp}=$2;
					}
				}

			}
		}	
		$content.="\n\n\\chapter{Glossary}\n\n";
		#while ( (my $key, my $value) = each(%glossarydefs) ) {
		$content.="\\begin{description}\n";
		foreach my $key (sort (keys(%glossarydefs))){
			$content.="\\item[".$key."] ".$glossarydefs{$key}."\n\n ";
		}
		$content.="\\end{description}\n";
			
		#}
	}
	return $wrapping.$content.$footer;


}

sub get_tex{
	my ($self, $pageobjectref)=@_;
	my @arr=@{$self->{_pageobjects}};
	return $arr[$pageobjectref]->pagecontenttex();
}
sub parse_for_text {
	my ($self,$start_node)=@_;
	my $returnvar=$start_node->attr('text');
	if($start_node->attr('_tag') eq 'img'){
		if($start_node->attr('alt')){
			$returnvar=$returnvar.$start_node->attr('alt');	
		} else {
				$returnvar=$returnvar." [img] ";
		}

	}
	my @arr=$start_node->content_list;
	foreach my $c(@arr){
		$returnvar=$returnvar.parse_for_text($self,$c);
	}
	return $returnvar;
}

sub parse_html_for_tex {
=head1  
parse_html_for_tex

Goes through the HTML nodes (HTML::Element/Tree) looking for anything familiar that it 'knows' what to do with (eg - to transfer it to TeX)
=cut

	my ($self,$start_node,$counter,$server_name)=@_;
	my $returnvar;
	

	if($start_node->attr('_tag') eq 'div' || $start_node->attr('_tag') eq 'p'){
		my $tempvar=$start_node->{'class'};
		my $tempvar2=$start_node->{'id'};
		my $results=0;
		my @whatever=@{$self->{_ignoredivs}};
		for (my $mytemp=0;$mytemp<$#whatever;$mytemp++){
			if($tempvar eq $whatever[$mytemp]){
				$results++;
			}
			if($tempvar2 eq $whatever[$mytemp]){
				$results++;
			}
		}
		if($results>0){
			# You've been told to ignore divs with this class name - so ignore them
			return $returnvar;
		}
	}

	# we only need to do all this in the body of the document	
	# (headers are examined for DC metadata elsewhere)
	
	if($start_node->is_inside('body')){
			if($start_node->attr('_tag') eq 'a'){
				
				my $tempvar=$start_node->attr('href');
				
				if($tempvar=~/http/ || $tempvar=~/\/(.*)/){

					my @arr=$start_node->content_list;

					foreach my $c(@arr){
						my $content=$returnvar." BBBBBB ".$self->parse_html_for_tex($c,$counter,$server_name)." AAAA";
						$content=~/(\w+\/)/;
						my $tmp=$1;
						$content=~s/$tmp/$tmp/g;
						$returnvar.=$content;
					}

					my $urlcontent="";
					my @childtag=$start_node->look_down('_tag','~text');

					if($#childtag>-1){
						$urlcontent=$childtag[0]->attr('text');
					}
						
					if($tempvar=~/http/){
						if($urlcontent eq $start_node->attr('href')){
							$returnvar.="  \\footnote{\\url{".$self->fix_tex_chars($start_node->attr('href'))."}}";

							} else {
							$returnvar.=" \\footnote{".$self->fix_long_uris($self->fix_tex_chars($urlcontent))." \\\n\\url{".$self->fix_tex_chars($start_node->attr('href'))."}}";
							}
						#alternative style: $returnvar.="$urlcontent [\\url{".$start_node->attr('href')."}] ";
					} else{
						$returnvar.=" \\footnote{".$self->fix_long_uris($self->fix_tex_chars($urlcontent))." \\url{".$server_name.$self->fix_tex_chars($start_node->attr('href'))."}}";
						#alternative style: $returnvar.="$urlcontent [\\url{".$server_name.$start_node->attr('href')."}]";

					}
					
				} else {
					my @arr=$start_node->content_list;
					foreach my $c(@arr){
						$returnvar=$returnvar.$self->parse_html_for_tex($c,$counter,$server_name);
					}
				}

			} elsif ($start_node->attr('_tag') eq 'b' | $start_node->attr('_tag') eq 'strong'){ # handles bold and strong
				$returnvar.= "{\\bf ";
				my @arr=$start_node->content_list;
				foreach my $c(@arr){
					$returnvar=$returnvar.$self->parse_html_for_tex($c,$counter,$server_name);
				}
				$returnvar.= "}";
			} elsif ($start_node->attr('_tag') eq 'i'){ 
				$returnvar.= "{\\em ";
				my @arr=$start_node->content_list;
				foreach my $c(@arr){
					$returnvar=$returnvar.$self->parse_html_for_tex($c,$counter,$server_name);
				}
				$returnvar.= "}";
			} elsif ($start_node->attr('_tag') eq 'p'){ 
				my @arr=$start_node->content_list;
				foreach my $c(@arr){
					$returnvar=$returnvar.$self->parse_html_for_tex($c,$counter,$server_name);
				}
				$returnvar.= "\n\n";
			} elsif($start_node->attr('_tag') eq 'abbr'){
				my @arr=$start_node->content_list;
				my $content;
				foreach my $c(@arr){
					$content=$returnvar.$self->parse_html_for_tex($c,$counter,$server_name);
					$returnvar.=$content;
				}
				if($self->{'_tex_useglossary'}>0){
					$returnvar.="\n\\nomenclature{$content}{".$start_node->attr('title')."}\n";
				}	
			} elsif ($start_node->attr('_tag') eq 'br'){
				$returnvar.= "\n\n";
			} elsif ($start_node->attr('_tag') eq 'img'){
				if($self->useimages>0){
					my $imgpath=$start_node->attr('src');
					$imgpath=~/\.(\w+)$/;
					my $imagext=$1;
					my $imgmd5=Digest::MD5::md5_hex($imgpath); 
					my $x; 
					my $y;
					if($imagext eq "gif"){
						($x,$y)=Image::Size::imgsize("temp/".$imgmd5.".gif");	
						if($x>400){
							 $returnvar.= " \n\\includegraphics[width=5in]{temp/".$imgmd5.".png}\n";
						} else {
							$returnvar.= " \n\\includegraphics[scale=1]{temp/".$imgmd5.".png}\n";

						}
					} elsif ($imagext eq "jpeg"){
					
					} else {
						($x,$y)=Image::Size::imgsize("temp/".$imgmd5.".$imagext");	
						if($x>400){
						$returnvar.= " \n\\includegraphics[width=5in]{temp/".$imgmd5.".".$imagext."}\n";
					 	} else {
						$returnvar.= " \n\\includegraphics{temp/".$imgmd5.".".$imagext."}\n";

						}
						# gifs don't work - need to be converted
					}
					if($start_node->attr('alt')){
						$returnvar.= "\n{\\em Caption: ".$start_node->attr('alt')."}\n\n";
					}
				}
			} elsif($start_node->attr('_tag') eq '~text'){
				my $text=$start_node->attr('text');
				my $content= $self->fix_long_uris($self->fix_tex_chars($text));
				$returnvar.=$content;
				my @arr=$start_node->content_list;
				foreach my $c(@arr){
					$returnvar=$returnvar.$self->parse_html_for_tex($c,$counter,$server_name);
				}
			} elsif($start_node->attr('_tag') eq 'script'){
			} elsif($start_node->attr('_tag') eq 'pre'){
				my @arr=$start_node->content_list;
				my $contenttext="";
				$returnvar=$returnvar."\n\\begin{verbatim*}\n".$self->parse_for_text($start_node)."\n\\end{verbatim*}";
				return $returnvar;
			} elsif($start_node->attr('_tag') eq 'acronym'){
				my @arr=$start_node->content_list;
				my $content;
				foreach my $c(@arr){
					$content=$returnvar.$self->parse_html_for_tex($c,$counter,$server_name);
					$returnvar.=$content;
				}
				if($self->{'_tex_useglossary'}>0){
					$returnvar.="\n\\nomenclature{$content}{".$start_node->attr('title')."}\n";
				}	
			} elsif($start_node->attr('_tag') eq 'table'){  

				# I am expecting: Table Tr [Th/Td] /Tr /Table
				my $tablecontent="";
				my @tablerows=$start_node->look_down('_tag','tr');
				my @tableimages=$start_node->look_down('_tag','img');
				my @tablebits;
				my $maxtablebits=0;
				my $tempint=0;
				foreach my $tablerow (@tablerows){
					my @tableheaders=$tablerow->look_down('_tag','th');
					my @tablecells=$tablerow->look_down('_tag','td');
					if(!defined @tablebits[$tempint]){
						@tablebits[$tempint]=();
					}
					foreach my $tablehead (@tableheaders){
							push(@{$tablebits[$tempint]},$self->fix_long_uris($self->fix_tex_chars($self->parse_for_text($tablehead))));
					}
					foreach my $tablehead (@tablecells){
							push(@{$tablebits[$tempint]},$self->fix_long_uris($self->fix_tex_chars($self->parse_for_text($tablehead))));
					}
					if($maxtablebits<$#tableheaders){
						$maxtablebits=$#tableheaders;
					}
					if($maxtablebits<$#tablecells){
						$maxtablebits=$#tablecells;
					}
					$tempint++;
				}
				if($maxtablebits>0){
					$returnvar.= "\n\n\\vspace{1em}\n{\\renewcommand{\\arraystretch}{1.5}\n\\setlongtables
\\setlength\\LTleft{0pt}
\\setlength\\LTright{50pt}
\\begin{longtable}{@\{\\extracolsep\{\\fill\}\}";
					my $textwdth=0.75/($maxtablebits+1);
					for(my $tempi=0;$tempi<=$maxtablebits;$tempi++){
						$returnvar.="|p{$textwdth\\textwidth}";
					}
					$returnvar.="|}\n\\hline\n";
							
					
					for (my $tempi=0;$tempi <= $#tablebits;$tempi++){
						for(my $tempj=0;$tempj <= $maxtablebits;$tempj++){
							$returnvar.= $tablebits[$tempi][$tempj];
							if($tempj<$maxtablebits){
								$returnvar.= " & ";
							}
							
						}
						$returnvar.="\\\\ \n\\hline\n";
					}
					$returnvar.= "\\end{longtable}}\n\n";
				} else {
      				for (my $tempi=0;$tempi <= $maxtablebits;$tempi++){
                        for(my $tempj=0;$tempj <= $#{$tablebits[$tempi]};$tempj++){
							$tablebits[$tempi][$tempj]=~s/[\r\n]/\n\n/;
                            $returnvar.= $tablebits[$tempi][$tempj]."";
                            if($tempj<$maxtablebits){
                                $returnvar.= " \n\n ";
                            }

                        }
					}
					$returnvar.=" \n\n";

				}
				foreach my $tableimage (@tableimages){
					$returnvar=$returnvar.$self->parse_html_for_tex($tableimage,$counter,$server_name); 	
				}
			} elsif($start_node->attr('_tag') =~ /[uo]l/ | $start_node->attr('_tag') eq 'dl'){    
				# go through the li 's 
				if($start_node->attr('_tag') eq 'ul'){
					$returnvar.="\n\n\\begin{enumerate}\n"; 
				} else {
					$returnvar.="\n\n\\begin{description}\n"; 

				}
				my @arr=$start_node->content_list;
				foreach my $c(@arr){
					$returnvar=$returnvar.$self->parse_html_for_tex($c,$counter,$server_name);
				}
				
				if($start_node->attr('_tag') eq 'ul'){
					$returnvar.="\\end{enumerate}\n"; 
				} else {
					$returnvar.="\\end{description}\n"; 

				}
			} elsif($start_node->attr('_tag') =~  /d([td])/){
				if($1=='t'){
					$returnvar.="\\item  "; 
				} else {
					$returnvar.="\\item  ";
				}
				
				my @arr=$start_node->content_list;
				foreach my $c(@arr){
					$returnvar=$returnvar.$self->parse_html_for_tex($c,$counter,$server_name);
				}
				$returnvar.="\n";
			} elsif($start_node->attr('_tag') eq  'li' ){
				$returnvar.= "\\item ";
				my @arr=$start_node->content_list;
				foreach my $c(@arr){
					$returnvar=$returnvar.$self->parse_html_for_tex($c,$counter,$server_name);
				}
				$returnvar.="\n";
				
			} elsif($start_node->attr('_tag') eq  'hr'){
				$returnvar.= "\\vspace{1em}\n\\hrule\n\n";
			} elsif($start_node->attr('_tag') =~ /h([0-9])/){

				my $subvar=$1;
				$returnvar.="\n\n\\";
				$subvar--;
				if($subvar>2){$subvar=2;}
				if($subvar>0){
					while($subvar>1){
						$returnvar.="sub";
						$subvar--;
					}
				}
				$returnvar.="section\{ ";
				# don't go through everything inside. Just find the child text node and use that
				my @sectiontext=$start_node->look_down('_tag','~text');
				foreach my $textitem (@sectiontext){
					$returnvar.=$self->fix_long_uris($self->fix_tex_chars($textitem->attr('text')));
				}

				$returnvar.=" }\n\n";
				
			} else {

				my @arr=$start_node->content_list;
				foreach my $c(@arr){
					$returnvar=$returnvar.$self->parse_html_for_tex($c,$counter,$server_name);
				}
				
			}
		} else {
			my @arr=$start_node->content_list;
			foreach my $c (@arr) {
				$returnvar=$returnvar.$self->parse_html_for_tex($c,$counter,$server_name);
			}

		}
		$counter++;
	
	return $returnvar;
}


sub parse_html_for_taglist {

=head1 
parse_html_for_taglist: Retrieve tags from HTML

You will probably never need to use this function directly - taking as an argument a HTML element tree (see HTML::Element and co), it steps through each tag looking for and returning tag names. 

=cut	
	#get all the tag names and return a flat list of them eg, 'html head meta meta meta '
	my ( $self, $start_node, $counter) = @_;
	
	my $returnvar;
 		$start_node->attr('_tag') if defined $start_node->attr('_tag');
		if(defined $start_node->attr('name')){
		}
		if(defined $start_node->attr('_tag')){
			print "Tag id: ".$start_node->attr('_tag')."\n";
			$returnvar.=" ".$start_node->attr('_tag');
		}
		$counter++;
	
	my @arr=$start_node->content_list;
	foreach my $c (@arr) {
		$returnvar=$returnvar." ".$self->parse_html_for_taglist($c,$counter) if ref $c; # ignore text nodes
	}
	return $returnvar;
}

sub title{
        my ( $self, $atitle) = @_;
        $self->{_title} = $atitle if defined($atitle);
        return $self->{_title};
}


sub get_html_content {

	my ( $self, $uri) = @_;

	my $browser=LWP::UserAgent->new;
	my $tf=HTML::TagFilter->new(allow=>{});
	my $tree = HTML::TreeBuilder->new();
	my $newpage=new PageObject();
	if(!defined($self->{_uri})){
		
	}elsif (defined($uri)){
		
	}elsif (defined($self->{_uri})){
		$uri=$self->{_uri};
	}	

	$newpage->uri($uri); 
	my $content = $browser->get($uri);
	die "Couldn't get $uri" unless defined $content;


	# Get headers from content hash 
	my $headers=$content->{'_headers'};

	# set date info from headers
	if($headers->{'date'}){
		$newpage->publish_date($headers->{'date'});
	}
	if($headers->{'last-modified'}){
		$newpage->updated($headers->{'last-modified'});
	}
	$tree->parse($content->content) || die "Can't find file\n";
		
	$tree->eof();
	my @title=$tree->look_down("_tag","title");
	my @basetag=$tree->look_down('_tag','base');
	foreach my $baset (@basetag){
		$newpage->base($baset->{'href'}); 
	}
	my @body=$tree->look_down('_tag','body');
	my $bod;
	$bod=@body[0]->{'_content'};
	 
	my $counter=0;
	my $parsed_html;

	# ' objectify text' makes text nodes from text data, called ~text and with an attr 'text'
	$tree->objectify_text();
	# extract server name from uri
	my $server_name;
	my @tempvar=split(/\//,$uri);
	$server_name="http://".$tempvar[2];
	$uri=~/^(.*)\/?/;	
	my $directory_name=$uri; 

	my @images=$tree->look_down('_tag','img');
	my $bits;
	my @imagesrc;
	my @imagemd5;
	my $urimd5=Digest::MD5::md5_hex($uri);
	foreach $bits (@images){
		$bits->{'src'}=~/\.(\w+)$/;
		my	$imageext=$1; 
		if(length($imageext)>1){
			$imageext=".".$imageext; 
			if($bits->{'src'} =~ /^http/){
				push(@imagesrc,$bits->{'src'});
				push(@imagemd5,Digest::MD5::md5_hex($bits->{'src'}).$imageext); 
			} else {
				if($newpage->base()){
					push(@imagesrc,$newpage->base().$bits->{'src'}); 
					push(@imagemd5,Digest::MD5::md5_hex($bits->{'src'}).$imageext); 
				} else {
					if($bits->{'src'}=~/^\//){
						push(@imagesrc,$server_name.$bits->{'src'}); 
						push(@imagemd5,Digest::MD5::md5_hex($bits->{'src'}).$imageext); 
					} else {
						push(@imagesrc,$directory_name.$bits->{'src'}); 
						push(@imagemd5,Digest::MD5::md5_hex($bits->{'src'}).$imageext); 

					}
				}

			}
		}
	}
	# Go download all the images into dir. 
 	if(! -d "temp"){	
		mkdir "temp" or die ("Couldn't create dir $urimd5"); 
	}
	my $imagecontent;
	for (my $count=0;$count<$#imagesrc;$count++){
		$imagecontent=$browser->get($imagesrc[$count]);
		
		open FILEHANDLE, ">./temp/".$imagemd5[$count];	
		print FILEHANDLE $imagecontent->content; 
		close FILEHANDLE; 
		my $command="convert ./temp/".$imagemd5[$count]." ./temp/".Digest::MD5::md5_hex($imagesrc[$count]).".png";
		my $returncode=`$command`;
	}	
	$newpage->images(@imagesrc);


	$newpage->pagecontenthtml($content->content);
	# Let's get all the meta tags (DC subject if available, etc)
	my @metavars=$tree->look_down("_tag" => "meta");

	my $metavar;
	foreach $metavar (@metavars){
		if($metavar->attr('name') eq "DC.Subject"){
			$newpage->subject($metavar->attr('content'));

		} elsif ($metavar->attr('name') eq "DC.Creator"){
			$newpage->author($metavar->attr('content'));	

		} elsif ($metavar->attr('name') eq "DC.Description"){
			$newpage->description($metavar->attr('content'));

		} elsif ($metavar->attr('name') eq "DC.Title"){
			$newpage->title($metavar->attr('content'));

		}

	}
	
	
	#	extract the title element from the page, set it as title 	
	my $title=$tf->filter($title[0]->as_HTML);
	chomp $title;
	$parsed_html=$self->parse_html_for_tex($tree,$counter,$server_name);
	$newpage->pagecontenttex($parsed_html);
	# set the title for the whole pdf?? probably best to do this manually?
	if($newpage->title() eq ""){
		$newpage->title($title);	# if we already set the title from the dc headers, no need to do it again here	
	}
	$newpage->uri($uri);	
	push @{$self->{_pageobjects}}, $newpage;
	$tree->delete;
}	

sub fix_tex_chars{
=head1 
fix_tex_chars: Escape TeX-sensitive chars 

This function offers a set of regular expressions to replace characters that will cause a TeX compiler to hiccup with something more appropriate. Use this after parsing HTML into TeX.
=cut 

	my($self,$sometex)=@_;
	# can't just do this to the lot. Just do it to those bits that aren't in verbatim
	$sometex=~s/\#/\\\#/g;
	$sometex=~s/\&/\\\&/g;
	$sometex=~s/\$/\\\$/g;
	$sometex=~s/\{/\\\{/g;
	$sometex=~s/\}/\\\}/g;
	$sometex=~s/\%/\\\%/g;
	$sometex=~s/\</\n\n\$\<\$/g;
	$sometex=~s/\>/\$\>\$/g;
	$sometex=~s/_/\_/g;
	$sometex=~s/\_/\\\_/g;
	return $sometex;
}

sub fix_long_uris{
	my ($self,$sometex)=@_;
	# find uri
	my $returnvar="";
	my @lines=split(/\n/,$sometex);
	foreach my $line (@lines){
		if($line=~/http\:+\/\/([\w\d\/\-_\*\.\,\+\&\?\\\=\~\%\:\;]*)/){
			# if($line=~/http\:\/\/(.*[^\$])/){
			my $tmp=$1;
			$tmp=~s/\\//g;
			# print "Line: $tmp \n";

			$line=~s/http\:\/\/$1/\\url{http\:\/\/$tmp}\ /;
			
			# $line=~s/http\:\/\/$1/\\\{\\url\ $1\ \}/;
			$returnvar=$returnvar.$line;
		} else{
			$returnvar=$returnvar.$line;
	
		}
		

	}
	return $returnvar;
}

package Frontend;

my $page=new HTMLtoTex;
$page->useimages(1);
$page->useglossary(1);
my @ignore=('navigation','qa-focus-header','qa-focus-footer','postscript');
$page->ignoredivs(@ignore);
for(my $tmp=1;$tmp<110;$tmp++){
	my $number=sprintf("%02d",$tmp);
	$page->get_html_content("http://www.ukoln.ac.uk/qa-focus/documents/briefings/briefing-$number/html/");

}

print $page->make_tex_all();
# vim: ft=perl
